<?php
/* Copyright (c) 2009, J.P.Westerhof <jurgen.westerhof@gmail.com>

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
 */

if(isset($_GET['show_source'])) {
    show_source(__FILE__);
    die();
}

require_once('levenshtein.php');
require_once('utf2html.php');

$from = 'The brown dog jumped away from the sprinkler';
if(isset($_POST['from']))
	$from = $_POST['from'];

$to = 'The dog ran towards the green sprinkler';
if(isset($_POST['to']))
	$to = $_POST['to'];

?><!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
   "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>
<title>Textual difference detector - a Levenshtein application example</title>
</head>
<body style="font-family: Lucida Console; font-size: 10pt;">
<h1>Textual difference detector</h1>
<table border="0" cellspacing="0" cellpadding="4" width="100%">
<tr><td style="width:408px;" valign="top">
<p>This is a Levenshtein application example discussed in the article '<a href="http://www.edesign.nl/2009/04/12/find-the-differences/">Find the differences</a>' published at <a href="http://www.edesign.nl">eDesign.nl</a>. Below is a form to enter test texts. Also links are included to the source code of the files needed for this example application.</p>
<form method="post" action="index.php">
	<p><b>Calculate transformations from:</b><br />
	<textarea name="from" style="width: 380px; height: 100px;" rows="5" cols="30"><?= utf2html($from);?></textarea><br />
	<b>Calculate transformations to:</b><br />
	<textarea name="to" style="width: 380px; height: 100px;" rows="5" cols="30"><?= utf2html($to);?></textarea><br />
	<input type="submit" value="Calculate" /></p>
</form>
<p style="font-size: 7pt;">Textual difference detector - a Levenshtein application example by <a href="http://www.edesign.nl">eDesign.nl</a><br />
Sources: <a href="?show_source">this file</a> | <a href="levenshtein.php?show_source">levenshtein.php</a> | <a href="utf2html.php?show_source">utf2html.php</a> (utf-8 helper)<br />

</p>
<p>
    <a href="http://validator.w3.org/check?uri=referer"><img
        src="http://www.w3.org/Icons/valid-xhtml10"
        alt="Valid XHTML 1.0 Strict" height="31" width="88" style="border: 0px;" /></a>
  </p>
  
</td> 
 <td valign="top" style="border-left: 1px solid black;"><div id="matched" style="width:100%">
<?php

$from = preg_split('/\s/', trim($from));
$to = preg_split('/\s/', trim($to));

echo '<h2>Levenshtein distance</h2>';
echo test_levenshtein($from, $to);

$lm = getLevenshteinMatrix($from, $to);

echo '<h2>Levenshtein matrix</h2><table>';
echo '<tr><td></td><td></td>';
foreach($from as $word)
	echo '<td style="padding: 3px;">'.utf2html($word).'</td>';
echo '</tr>';
$count = 0;
foreach($lm as $row) {
	echo '<tr>';
	echo '<td>';
	if(isset($to[$count - 1]))
		echo utf2html($to[$count - 1]);
	echo '</td>';
	foreach($row as $cell) {
		echo '<td align="center" style="padding: 3px;">'.$cell.'</td>';
	}
	echo '</tr>';
	$count++;
}
echo '</table>';



echo '<h2>Visualized transformations</h2>';

$trans = getTransformations($from, $to);
foreach($trans as $t) {
	if($t[0] == 'unchanged')
		echo utf2html($t[1]);
	elseif($t[0] == 'deleted')
		echo '<span style="color: #ff0000">'.utf2html($t[1]).'</span>';
	elseif($t[0] == 'added')
		echo '<span style="color: #008800">'.utf2html($t[1]).'</span>';
	echo ' ';
}


echo '<h2>Merged transformations</h2>';

$trans = mergeTransformations($trans);
foreach($trans as $t) {
	if($t[0] == 'unchanged')
		echo utf2html($t[1]);
	elseif($t[0] == 'deleted')
		echo '<span style="color: #ff0000">'.utf2html($t[1]).'</span>';
	elseif($t[0] == 'added')
		echo '<span style="color: #008800">'.utf2html($t[1]).'</span>';
	echo ' ';
}
?></div></td>
</tr>

</table>
</body>
</html>